@@ -626,7 +626,7 @@ module Nokogiri
 
         # issue #1821, #2110
         def test_parse_can_take_pathnames
-          assert(File.size(HTML_FILE) > 4096) # file must be big enough to trip the read callback more than once
+          assert_operator(File.size(HTML_FILE), :>, 4096) # file must be big enough to trip the read callback more than once
 
           doc = Nokogiri::HTML4.parse(Pathname.new(HTML_FILE))
 
