@@ -3,9 +3,14 @@
 require "helper"
 
 describe "compaction" do
+  def skip_compaction_tests
+    # supported in CRuby >= 3.0
+    !GC.respond_to?(:verify_compaction_references)
+  end
+
   describe Nokogiri::XML::Node do
     it "compacts safely" do # https://github.com/sparklemotion/nokogiri/pull/2579
-      skip unless GC.respond_to?(:verify_compaction_references)
+      skip if skip_compaction_tests
 
       big_doc = "<root>" + ("a".."zz").map { |x| "<#{x}>#{x}</#{x}>" }.join + "</root>"
       doc = Nokogiri.XML(big_doc)
@@ -23,7 +28,7 @@ describe "compaction" do
 
   describe Nokogiri::XML::Namespace do
     it "namespace_scopes" do
-      skip unless GC.respond_to?(:verify_compaction_references)
+      skip if skip_compaction_tests
 
       doc = Nokogiri::XML(<<~EOF)
         <root xmlns="http://example.com/root" xmlns:bar="http://example.com/bar">
@@ -41,7 +46,7 @@ describe "compaction" do
     end
 
     it "remove_namespaces!" do
-      skip unless GC.respond_to?(:verify_compaction_references)
+      skip if skip_compaction_tests
 
       doc = Nokogiri::XML(<<~XML)
         <root xmlns:a="http://a.flavorjon.es/" xmlns:b="http://b.flavorjon.es/">
