@@ -3,30 +3,29 @@
 
 require "helper"
 
-class TestEncodingHandler < Nokogiri::TestCase
-  def teardown
+describe Nokogiri::EncodingHandler do
+  after do
     Nokogiri::EncodingHandler.clear_aliases!
-    # Replace default aliases removed by clear_aliases!
-    Nokogiri.install_default_aliases
-    super
+    Nokogiri::EncodingHandler.install_default_aliases
   end
 
-  def test_get
+  it :test_get do
+    refute_nil(Nokogiri::EncodingHandler["NOKOGIRI-SENTINEL"])
     refute_nil(Nokogiri::EncodingHandler["UTF-8"])
     assert_nil(Nokogiri::EncodingHandler["alsdkjfhaldskjfh"])
   end
 
-  def test_name
+  it :test_name do
     eh = Nokogiri::EncodingHandler["UTF-8"]
     assert_equal("UTF-8", eh.name)
   end
 
-  def test_alias
+  it :test_alias do
     Nokogiri::EncodingHandler.alias("UTF-8", "UTF-18")
     assert_equal("UTF-8", Nokogiri::EncodingHandler["UTF-18"].name)
   end
 
-  def test_cleanup_aliases
+  it :test_cleanup_aliases do
     assert_nil(Nokogiri::EncodingHandler["UTF-9"])
     Nokogiri::EncodingHandler.alias("UTF-8", "UTF-9")
     refute_nil(Nokogiri::EncodingHandler["UTF-9"])
@@ -35,7 +34,7 @@ class TestEncodingHandler < Nokogiri::TestCase
     assert_nil(Nokogiri::EncodingHandler["UTF-9"])
   end
 
-  def test_delete
+  it :test_delete do
     assert_nil(Nokogiri::EncodingHandler["UTF-9"])
     Nokogiri::EncodingHandler.alias("UTF-8", "UTF-9")
     refute_nil(Nokogiri::EncodingHandler["UTF-9"])
@@ -44,7 +43,13 @@ class TestEncodingHandler < Nokogiri::TestCase
     assert_nil(Nokogiri::EncodingHandler["UTF-9"])
   end
 
-  def test_delete_non_existent
+  it :test_delete_non_existent do
     assert_nil(Nokogiri::EncodingHandler.delete("UTF-9"))
   end
+
+  it "deprecates Nokogiri.install_default_aliases" do
+    assert_output("", /deprecated/) do
+      Nokogiri.install_default_aliases
+    end
+  end
 end
