@@ -17,8 +17,8 @@ module Nokogiri
 
       def test_comment?
         comment = Nokogiri::XML::Comment.new(@xml, "hello world")
-        assert(comment.comment?)
-        refute(@xml.root.comment?)
+        assert_predicate(comment, :comment?)
+        refute_predicate(@xml.root, :comment?)
       end
 
       def test_passing_a_node_uses_the_node_document
